@@ -1,6 +1,7 @@
 import numpy as np
 import torch
 import datasets
+import datasets as ds
 from datasets import load_dataset, Dataset
 from transformers import DefaultDataCollator
 from transformers import DataCollatorForTokenClassification
@@ -17,6 +18,7 @@ import re
 from transformers.tokenization_utils_base import PreTrainedTokenizerBase
 import inspect
 load_dataset = lazy_func(datasets.load_dataset)
+from scipy.special import expit
 
 def get_dataset_name(dataset):
     try:
@@ -29,7 +31,7 @@ def sample_dataset(dataset,n=10000, n_eval=1000):
     for k in dataset:
         n_k=(n if k=='train' else n_eval)
         if n_k and len(dataset[k])>n_k:
-            dataset[k]=dataset[k].select(range(n_k))
+            dataset[k]=dataset[k].train_test_split(train_size=n_k)['train']
     return dataset
 
 @dataclass
@@ -60,6 +62,8 @@ class Task:
             self.name = name
         self.dataset=sample_dataset(self.dataset,self.max_rows,self.max_rows_eval)
 
+    def check():
+        return True
 
     def set_tokenizer(self, tokenizer):
         self.tokenizer = tokenizer
@@ -84,7 +88,22 @@ class Classification(Task):
             elif hasattr(target,'num_classes'):
                 self.num_labels=target.num_classes
             else:
-                self.num_labels=len(set(self.dataset['train'][self.y]))
+                self.num_labels=max(fc.flatten(self.dataset['train'][self.y]))+1
+
+        if type(self.dataset['train'][self.y][0])==list:
+            self.problem_type="multi_label_classification"
+            if set(fc.flatten(self.dataset['train'][self.y]))!={0,1}:
+                def one_hot(x):
+                    x[self.y] = [float(i in x[self.y]) for i in range(self.num_labels)]
+                    return x
+                self.dataset=self.dataset.map(one_hot)
+            
+            self.num_labels=len(self.dataset['train'][self.y][0])
+            self.dataset=self.dataset.cast_column(self.y, ds.Sequence(feature=ds.Value(dtype='float64')))
+
+    def check(self):
+        features = self.dataset['train'].features
+        return self.s1 in features and self.y in features
 
     def preprocess_function(self, examples):
         inputs = (
@@ -97,14 +116,21 @@ class Classification(Task):
         return outputs
 
     def compute_metrics(self, eval_pred):
+        avg={}
         predictions, labels = eval_pred.predictions, eval_pred.label_ids
         if "int" in str(eval_pred.label_ids.dtype):
             metric = load_metric("super_glue", "cb")
             predictions = np.argmax(predictions, axis=1)
+            
+        elif getattr(self,"problem_type", None)=='multi_label_classification':
+            metric=evaluate.load('f1','multilabel', average='macro')
+            labels=labels.astype(int)
+            predictions = (expit(predictions)>0.5).astype(int)
+            avg={"average":"macro"}
         else:
             metric = load_metric("glue", "stsb")
         meta = {"name": self.name, "size": len(predictions), "index": self.index}
-        return {**metric.compute(predictions=predictions, references=labels), **meta}
+        return {**metric.compute(predictions=predictions, references=labels,**avg), **meta}
 
 
 @dataclass
@@ -149,9 +175,14 @@ class MultipleChoice(Classification):
         choices = [x for x in self.dataset['train'].features if re.match('choice\d+',x)]
         if choices and not self.choices:
             self.choices=choices
+        self.num_choices = len(self.choices)
     def set_tokenizer(self, tokenizer):
         self.tokenizer = self.data_collator.tokenizer= tokenizer
 
+    def check(self):
+        features = self.dataset['train'].features
+        return self.s1 in features and self.y in features and self.choices and all([c in features for c in self.choices])
+
     def preprocess_function(self, examples):
         num_choices = len(self.choices)
         num_examples = len(examples[self.s1])
@@ -183,8 +214,8 @@ class TokenClassification(Task):
     dataset: Dataset = None
     metric:... = evaluate.load("seqeval")
 
-    tokens: str = None
-    y: str = None
+    tokens: str = 'tokens'
+    y: str = 'labels'
     num_labels: int = None
 
     @staticmethod
@@ -261,6 +292,9 @@ class TokenClassification(Task):
             **meta,
         }
 
+    def check(self):
+        features = self.dataset['train'].features
+        return self.tokens in features and self.y in features
 
 @dataclass
 class Seq2SeqLM(Task):
